.include "m8515def.inc"

.def temp = R16
.equ CLOCK = 12000000
.equ BAUD = 9600
.equ UBRRVAL = CLOCK/(BAUD*16)-1


;Display.asm **********************************************************************************************************
.def LED_Value_Hex = r20		;Hexadezimale Anzeige
.def LED_Delay_ValueH = r30		;Zeit zum Warten (High und Low von LED_DelayXXms)
.def LED_Delay_ValueL = r29
.def LED_Delay_ValueM = r28		;Multiplikator fr's warten
.def LED_Temp = r16				;Temporre Register
.def LED_Temp2 = r17
.def LED_Delay_SetLed = r18		;= 1, wenn beim Warten ebenfalls angezeigt werden soll

.equ LED_Delay1ms = 1000		;1ms warten = 12000 Zyklen nichts tun
.equ LED_Delay5ms = 5000		;5ms warten = 60000 Zyklen nichts tun
.equ LED_Delay10ms = 10000		;10ms warten = 120000 Zyklen nichts tun
.equ LED_Delay30ms = 30000		;30ms warten = 360000 Zyklen nichts tun
.equ LED_Delay60ms = 60000		;60ms warten = 720000 Zyklen nichts tun

.equ LED_Display_0 = 0b00111111
.equ LED_Display_1 = 0b00000110
.equ LED_Display_2 = 0b01011011
.equ LED_Display_3 = 0b01001111
.equ LED_Display_4 = 0b01100110
.equ LED_Display_5 = 0b01101101
.equ LED_Display_6 = 0b01111101
.equ LED_Display_7 = 0b00000111
.equ LED_Display_8 = 0b01111111
.equ LED_Display_9 = 0b01101111
.equ LED_Display_A = 0b01110111
.equ LED_Display_B = 0b01111100
.equ LED_Display_C = 0b00111001
.equ LED_Display_D = 0b01011110
.equ LED_Display_E = 0b01111001
.equ LED_Display_F = 0b01110001
;**********************************************************************************************************************





.org 0x00
        rjmp main

.org URXCaddr                             ; Interruptvektor fr UART-Empfang
        rjmp int_rxc

; Hauptprogramm
main:
        ldi temp, LOW(RAMEND)
        out SPL, temp
        ldi temp, HIGH(RAMEND)
        out SPH, temp

        ; Baudrate einstellen
        ldi temp, LOW(UBRRVAL)
        out UBRRL, temp
        ldi temp, HIGH(UBRRVAL)
        out UBRRH, temp

        ; Frame-Format: 8 Bit
        ldi temp, (1<<URSEL)|(1<<UCSZ0)	  ;= 0b10000000 Or 0b110 (URSEL = 7, UCSZ0 = 1)
        out UCSRC, temp

        sbi UCSRB, RXCIE                  ; Interrupt bei Empfang
        sbi UCSRB, RXEN                   ; RX (Empfang) aktivieren
        
        ldi temp, 0x0F
        out DDRA, temp
        
        rcall LED_Initialize
        rcall LED_Test
        
        sei                               ; Interrupts global aktivieren
		
		loop:   
			rcall LED_Out_Hex
			
			rcall lED_Delay_10ms
		rjmp loop                         ; Endlosschleife
		

; Interruptroutine: wird ausgefhrt sobald ein Byte ber das UART empfangen wurde
int_rxc:
		push temp
		push r17

        in temp, UDR
        
        ldi r17, 0xFF
		
		sbis PINA, 4
		eor temp, r17
		
		sbis PINA, 5
		swap temp
		
		sbis PINA, 6
		rol temp
		
		sbis PINA, 7
		ror temp
		
		mov LED_Value_Hex, temp
		        
        pop r17
        pop temp
reti                              		  ; Interrupt beenden








;Display.asm **********************************************************************************************************








LED_Initialize:					;Anzeige initialisieren
	
	push LED_Temp				;Register speichern
	
	ldi LED_Temp, 0xFF
	out DDRC, LED_Temp			;Richtungsregister konfigurieren: PortC = Ausgang
	
	ldi LED_Temp, 0b00000011
	out DDRE, LED_Temp			;Bit 0 und 1 von PortE sind Ausgnge
	
	ldi LED_Temp, 0xFF
	out PORTC, LED_Temp			;Anzeige ausschalten
	
	ldi LED_Temp, 0x01
	out PORTE, LED_Temp
	
	pop LED_Temp
	
;	rcall LED_Test				;Displaytest
;	
;	ldi LED_Temp, 0xFF
;	ldi LED_Delay_SetLed, 1		;Beim Warten auch ausgeben!
;	ldi LED_Value_Hex, 0		;Bei 0 zu Zhlen anfangen
;
;	LED_Ini_Loop_:
;		
;		rcall LED_Delay_180ms	;180ms warten
;		
;		inc LED_Value_Hex
;		
;		sbis PINA, 4
;		ldi LED_Value_Hex, 0xFF
;		
;		rcall LED_Out_Hex		;Wert ausgeben
;		
;		cpi LED_Value_Hex, 0xFF
;		brne LED_Ini_Loop_
;		
;		sbic PINA, 4
;		eor LED_Value_Hex, LED_Value_Hex
;	
;	rjmp LED_Ini_Loop_			;Endlosschleife

ret
	
	
	
LED_Test:						;Einen Displaytest durchfhren

	push LED_Temp				;Register speichern
	push LED_Temp2
	push r19
	
	ldi LED_Delay_SetLed, 0		;Beim Warten nicht ausgeben!
	
	ldi LED_Temp, 1				;0x00000001 ins Register laden
	ldi r19, 0xFF
	
	;Erste Stelle testen
	ldi LED_Temp2, 0b10
	out PORTE, LED_Temp2
	
	LED_Test_Loop_:
	
		eor LED_Temp, r19		;Register umkehren		
		out PORTC, LED_Temp		;Wert ausgeben		
		eor LED_Temp, r19		;Register wieder zurckkehren
	
		sbrc LED_Temp, 7		;Ist das 7. Bit gesetzt?
		rjmp LED_Test_2nd_		;Ja, Schleife verlassen
	
		lsl LED_Temp			;Ein Bit nach links schieben
	
		rcall LED_Delay_120ms	;120ms warten	
	
	rjmp LED_Test_Loop_		
	
	LED_Test_2nd_:
	
	rcall LED_Delay_120ms		;120ms warten	
	
	
	ldi LED_Temp, 1
	
	;Zweite Stelle testen
	ldi LED_Temp2, 0b01
	out PORTE, LED_Temp2
	
	LED_Test_Loop2_:
	
		eor LED_Temp, r19		;Register umkehren		
		out PORTC, LED_Temp		;Wert ausgeben		
		eor LED_Temp, r19		;Register wieder zurckkehren
	
		sbrc LED_Temp, 7		;Ist das 7. Bit gesetzt?
		rjmp LED_Test_End_		;Ja, Schleife verlassen
	
		lsl LED_Temp			;Ein Bit nach links schieben
	
		rcall LED_Delay_120ms	;120ms warten	
	
	rjmp LED_Test_Loop2_		
	
	
	LED_Test_End_:

	
	pop r19
	pop LED_Temp2
	pop LED_Temp
		
ret


LED_Out_Hex:					;Den Hex-Wert in LED_Value_Hex anzeigen

	push LED_Temp				;Register speichern
	push LED_Temp2
	
	;Von der einen auf die andere Stelle umschalten
	in LED_Temp, PORTE
	
	;Ist da vielleicht 0b11 oder 0b00 drin?
	cpi LED_Temp, 0
	breq LED_Out_Hex_SetE_
	cpi LED_Temp, 3
	brsh LED_Out_Hex_SetE_
	rjmp LED_Out_Hex_Continue4_
	
	LED_Out_Hex_SetE_:
		ldi LED_Temp, 1		;Ja, auf 0b01 setzen
	
	LED_Out_Hex_Continue4_:
	
	sbrs LED_Temp, 0
	rjmp LED_Out_Hex_SR_
		lsl LED_Temp
		rjmp LED_Out_Hex_Continue_
	LED_Out_Hex_SR_:
		lsr LED_Temp
	LED_Out_Hex_Continue_:

	out PORTE, LED_Temp
	
	;Den auszugebenden Wert fr die aktuelle Anzeige in LED_Temp laden
	mov LED_Temp, LED_Value_Hex
	sbis PORTE, 0
	rjmp LED_Out_Hex_Continue2_
		swap LED_Temp
	LED_Out_Hex_Continue2_:
	andi LED_Temp, 0x0F


	cpi LED_Temp, 0x00
	breq LED_Out_Hex_0x00_
	cpi LED_Temp, 0x01
	breq LED_Out_Hex_0x01_
	cpi LED_Temp, 0x02
	breq LED_Out_Hex_0x02_
	cpi LED_Temp, 0x03
	breq LED_Out_Hex_0x03_
	cpi LED_Temp, 0x04
	breq LED_Out_Hex_0x04_
	cpi LED_Temp, 0x05
	breq LED_Out_Hex_0x05_
	cpi LED_Temp, 0x06
	breq LED_Out_Hex_0x06_
	cpi LED_Temp, 0x07
	breq LED_Out_Hex_0x07_
	cpi LED_Temp, 0x08
	breq LED_Out_Hex_0x08_
	cpi LED_Temp, 0x09
	breq LED_Out_Hex_0x09_
	cpi LED_Temp, 0x0A
	breq LED_Out_Hex_0x0A_
	cpi LED_Temp, 0x0B
	breq LED_Out_Hex_0x0B_
	cpi LED_Temp, 0x0C
	breq LED_Out_Hex_0x0C_
	cpi LED_Temp, 0x0D
	breq LED_Out_Hex_0x0D_
	cpi LED_Temp, 0x0E
	breq LED_Out_Hex_0x0E_
	cpi LED_Temp, 0x0F
	breq LED_Out_Hex_0x0F_
	
	LED_Out_Hex_0x00_:
	ldi LED_Temp2, LED_Display_0
	rjmp LED_Out_Hex_Continue3_
	LED_Out_Hex_0x01_:
	ldi LED_Temp2, LED_Display_1
	rjmp LED_Out_Hex_Continue3_
	LED_Out_Hex_0x02_:
	ldi LED_Temp2, LED_Display_2
	rjmp LED_Out_Hex_Continue3_
	LED_Out_Hex_0x03_:
	ldi LED_Temp2, LED_Display_3
	rjmp LED_Out_Hex_Continue3_
	LED_Out_Hex_0x04_:
	ldi LED_Temp2, LED_Display_4
	rjmp LED_Out_Hex_Continue3_
	LED_Out_Hex_0x05_:
	ldi LED_Temp2, LED_Display_5
	rjmp LED_Out_Hex_Continue3_
	LED_Out_Hex_0x06_:
	ldi LED_Temp2, LED_Display_6
	rjmp LED_Out_Hex_Continue3_
	LED_Out_Hex_0x07_:
	ldi LED_Temp2, LED_Display_7
	rjmp LED_Out_Hex_Continue3_
	LED_Out_Hex_0x08_:
	ldi LED_Temp2, LED_Display_8
	rjmp LED_Out_Hex_Continue3_
	LED_Out_Hex_0x09_:
	ldi LED_Temp2, LED_Display_9
	rjmp LED_Out_Hex_Continue3_
	LED_Out_Hex_0x0A_:
	ldi LED_Temp2, LED_Display_A
	rjmp LED_Out_Hex_Continue3_
	LED_Out_Hex_0x0B_:
	ldi LED_Temp2, LED_Display_B
	rjmp LED_Out_Hex_Continue3_
	LED_Out_Hex_0x0C_:
	ldi LED_Temp2, LED_Display_C
	rjmp LED_Out_Hex_Continue3_
	LED_Out_Hex_0x0D_:
	ldi LED_Temp2, LED_Display_D
	rjmp LED_Out_Hex_Continue3_
	LED_Out_Hex_0x0E_:
	ldi LED_Temp2, LED_Display_E
	rjmp LED_Out_Hex_Continue3_
	LED_Out_Hex_0x0F_:
	ldi LED_Temp2, LED_Display_F
	
	LED_Out_Hex_Continue3_:


	ldi LED_Temp, 0xFF				;Wert umkehren (0 = Ein, 1 = Aus)
	eor LED_Temp2, LED_Temp
	
	out PORTC, LED_Temp2			;Wert ausgeben
	
	pop LED_Temp2
	pop LED_Temp
	
ret




LED_Delay_180ms:
	ldi LED_Delay_ValueH, HIGH(LED_Delay60ms)
	ldi LED_Delay_ValueL, LOW(LED_Delay60ms)
	ldi LED_Delay_ValueM, 3
	
	rcall LED_Delay
ret

LED_Delay_120ms:
	ldi LED_Delay_ValueH, HIGH(LED_Delay60ms)
	ldi LED_Delay_ValueL, LOW(LED_Delay60ms)
	ldi LED_Delay_ValueM, 2
	
	rcall LED_Delay
ret

LED_Delay_10ms:
	ldi LED_Delay_ValueH, HIGH(LED_Delay10ms)
	ldi LED_Delay_ValueL, LOW(LED_Delay10ms)
	ldi LED_Delay_ValueM, 1
	
	rcall LED_Delay
ret

	
LED_Delay:						;Eine bestimmte Zeit warten (Zeit in LED_Delay_Value wird gewartet)

	push LED_Temp				;Register speichern
	push LED_Temp2
	
	mov LED_Temp, LED_Delay_ValueL	;Low-Value speichern
	mov LED_Temp2, LED_Delay_ValueH ;High-Value speichern

	LED_Delay_LoopM_:
	
		mov LED_Delay_ValueH, LED_Temp2		;Zhlregister wieder fllen

		LED_Delay_Loop_:
		
			mov LED_Delay_ValueL, LED_Temp	;Zhlregister wieder fllen
			
			LED_Delay_Loop2_:
			
				dec LED_Delay_ValueL		;Register dekrementieren (- 1)
				
				nop
				nop
				nop
				
				nop
				nop
				nop
				nop
				nop
				nop
				
				cpi LED_Delay_ValueL, 0		;Ist es 0?
				
				brne LED_Delay_Loop2_		;Wenn nicht, dann Schleife
				
			

			dec LED_Delay_ValueH			;Register dekrementieren (- 1)
			
			cpi LED_Delay_ValueH, 0			;Ist es 0?
			
			brne LED_Delay_Loop_			;Wenn nicht, dann Schleife
		
		
		dec LED_Delay_ValueM			;Register dekrementieren (- 1)
		
		cpi LED_Delay_ValueM, 0			;Ist es 0?
		
		brne LED_Delay_LoopM_			;Wenn nicht, dann Schleife

	
	pop LED_Temp2
	pop LED_Temp
	
ret								;Prozedurende
